import os
import sys 
import json
import imp

from copy import copy

from importlib import import_module
import inflect

from jinja2 import Environment, PackageLoader
from finny.command import Command

class GenerateEndpoint(Command):

  def __init__(self):
    self.pluralize = inflect.engine()

  def _touch(self, filepath):
    open(filepath, 'a').close()


  def _transform_param_names(self):
    name = self.params.name
    self.params.original_name = name

    names = [ i for i in name.split("_") ]

    plural_names = copy(names)
    plural_names[-1] = self.pluralize.plural(plural_names[-1])

    self.params.name = "".join([ n.capitalize() for n in names])
    self.params.plural_name = "".join([ n.capitalize() for n in plural_names])
    self.params.plural_orig_name = "_".join(plural_names)


  def _read_app_name(self):
    cwd = os.getcwd()

    config = imp.new_module('config')
    config.__file__ = "config"
    # loads the configuration file
    with open(cwd + "/__init__.py") as config_file:
      exec(compile(config_file.read(), "config", 'exec'), config.__dict__)

    self.app_name = config.__APP__

  def run(self, params):
    self.params = params

    cwd = os.getcwd()

    self._read_app_name()
    self._transform_param_names()

    endpoint_path = cwd + "/resources/" + self.params.original_name
    # create folder for endpoint
    os.makedirs(endpoint_path)

    # copy templates over
    self._copy_templates([ "api.py", "model.py"], "endpoint", endpoint_path)

    self.read_default_runner()

  def read_default_runner(self):
    cwd = os.getcwd() + "/resources"

    endpoints = [ name for name in os.listdir(cwd) if os.path.isdir(os.path.join(cwd, name)) ]

    cwd = os.getcwd()

    with open("%s/%s/runners/default.py" % (cwd, self.app_name), "w+") as f:
      f.write("ENDPOINTS = %s" % json.dumps(endpoints))

  def _copy_templates(self, source, src, dst):
    env = Environment(loader=PackageLoader('finny.commands', 'templates/' + src))

    for item in source:
      template = env.get_template("%s.jinja" % item)
      output = template.render(name=self.params.name,
                               plural_name=self.params.plural_name,
                               app_name=self.app_name,
                               original_name=self.params.original_name,
                               plural_orig_name=self.params.plural_orig_name)

      path = dst + "/" + item
      dirname = os.path.dirname(path)

      if not os.path.exists(dirname):
        os.makedirs(dirname)

      self._touch(dirname + "/__init__.py")

      with open(path, "w+") as f:
        f.write(output)



import os
import sys
import argparse

import imp

from finny.commands import CommandFactory

STRUCTURE_REPRESENTATIVES = [ "manage.py", "__init__.py" ]

class CLI(object):

  def __init__(self):
    self.parser = argparse.ArgumentParser(description='Finny.')
    self.subparsers = self.parser.add_subparsers(help='sub-command help')

  def add_subparsers(self, *args, **kwargs):
    return self.parser.add_subparsers(*args, **kwargs)

  def add_param(self, *args, **kwargs):
    return self.parser.add_argument(*args, **kwargs)

  def add_argument(self, *args, **kwargs):
    return self.parser.add_argument(*args, **kwargs)

  def run(self):
    self.params = self.parser.parse_args()
    self.main()

class Finny(CLI):
  """
  Finny entry point
  """
  def __init__(self, is_structure):
    super(Finny, self).__init__()

    self.is_structure = is_structure

    self.parser = argparse.ArgumentParser(description='Process some integers.')

  def run_command(self):
    command = CommandFactory.get(command=self.params.command,
                                 command_type=self.params.command_type)
    command.run(self.params)

  def generate_structure(self):
    path = self.params.path

    if path[0] != "/":
      path = os.path.abspath(os.getcwd() + "/" + self.params.path)

    name = path.split("/")[-1]

    if os.path.exists(path):
      raise AttributeError("Path %s is already present." % path)

    CommandFactory.get(command="GenerateStructure").run(name, path)

  def main(self):
    if self.is_structure:
      self.run_command()
    else:
      self.generate_structure()

def detect_current_structure():
  current_path = os.getcwd()

  reprensentatives = [ os.path.exists(current_path + "/" + item)
                      for item in STRUCTURE_REPRESENTATIVES ]

  if all(reprensentatives):
    config = imp.new_module('config')
    config.__file__ = "config"
    # loads the configuration file
    with open(current_path + "/__init__.py") as config_file:
      exec(compile(config_file.read(), "config", 'exec'), config.__dict__)

    finny_app = config.__APP__

    if os.path.exists(current_path + "/" + finny_app):
      return True

  return False

def execute_from_cli():
  current_path = os.getcwd()
  is_structure = detect_current_structure()

  f = Finny(is_structure)

  if is_structure:
    subparsers = f.add_subparsers(dest="command",
                                  help='Commands for a present finny app')

    parser_generate = subparsers.add_parser('generate',
                                        help='Generate a number of constructs')

    subparser_gen = parser_generate.add_subparsers(dest="command_type",
                                        help='Generate a number of constructs')

    parser_runner = subparser_gen.add_parser('runner',
                                             help='Generate new runner')

    parser_endpoint = subparser_gen.add_parser('endpoint',
                                               help='Generate new endpoint')

    parser_default_runner  = subparser_gen.add_parser('default-runner',
                                                    help='Generate new runner')

    parser_runner.add_argument("name", help="Name of the runner")
    parser_endpoint.add_argument("name", help="Name of the endpoint")

  else:
    subparsers = f.add_subparsers(help='sub-command help')
    parser_new = subparsers.add_parser('new', help='new help')
    parser_new.add_argument("path", help="Path or name of the finny app",
                             default=False)

  f.run()



import os
from importlib import import_module

import inflect

from flask import url_for

from finny.views import ResourceBuilder

STAGES = [ "development", "test", "production" ]

def _pluralize(name):
  pluralize = inflect.engine()
  names = [ i for i in name.split("_") ]
  names[-1] = pluralize.plural(names[-1])

  plural_name = "".join([ n.capitalize() for n in names])
  return plural_name

def load_runner(name, app, runner):
  """
  For each endpoint in the runner load the class
  """
  # loads the runner that has a list of endpoints
  module = import_module("%s.runners.%s" % (name, runner))

  for endpoint in module.ENDPOINTS:
    endpoint_name = endpoint
    except_methods = []

    if type(endpoint) == dict:
      endpoint_name = endpoint["view"]
      except_methods = endpoint["except"]

    module = import_module("resources.%s.api" % endpoint_name)
    plural_name = _pluralize(endpoint_name)
    klass = getattr(module, "%sView" % plural_name)

    klass.except_methods = except_methods
    # TODO: Think at something better that this hardcoded prefix
    klass.register()

  ResourceBuilder().build(app)

  if runner == "default":

    # TODO: Remove this HACK!
    @app.errorhandler(404)
    def page_not_found(e):
      import urllib
      output = ""
      for rule in app.url_map.iter_rules():

          options = {}
          for arg in rule.arguments:
              options[arg] = "[{0}]".format(arg)

          methods = ','.join(rule.methods)
          url = url_for(rule.endpoint, **options)
          line = urllib.unquote("{:50s} {:20s} {}".format(rule.endpoint, methods, url))
          line = "<strong>%s</strong> %s %s" % (rule.endpoint, methods, urllib.unquote(url))
          output += "<li>" + line + "</li>"

      return """
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<title>404 Not Found</title>
<h1>Not Found</h1>
<p>The requested URL was not found on the server.  If you entered the URL manually please check your spelling and try again.</p>

<h3>Current routes:</h3>
<ul>
%s
</ul>
      """ % output, 404

def load_environment_config(app, env):
  if env not in STAGES:
    raise AttributeError("Stage %s must be in %s" % (env, STAGES))

  config = os.path.join(app.root_path, '%s.py' % env)
  app.config.from_pyfile(config)



import os
import imp

def read_env_variable(variable_name):

  if variable_name in os.environ:
    return os.environ[variable_name]
  else:
    current_path = os.getcwd()

    config = imp.new_module('config')
    config.__file__ = "config"

    if os.path.exists(current_path + "/.env"):
      # loads the configuration file
      with open(current_path + "/.env") as config_file:
        exec(compile(config_file.read(), "config", 'exec'), config.__dict__)

      if hasattr(config, variable_name):
        return getattr(config, variable_name)

    raise AttributeError("Could not find %s as an ENV variable or in the .env file")



from .generate_structure import GenerateStructure
from .generate_runner import GenerateRunner
from .generate_endpoint import GenerateEndpoint 

class CommandFactory(object):

  @staticmethod
  def get(command, command_type=None):
    if command == "GenerateStructure":
      return GenerateStructure()

    if command == "generate":
      if command_type == "runner":
        return GenerateRunner()

      if command_type == "endpoint":
        return GenerateEndpoint()

    raise NotImplementedError("The command with command=%s and command_type=%s" \
                              % (command, command_type))



#def fixtures_wrapper(app_name):
#
#  def fixtures(module, yaml_file):
#
#    yaml_file = Path.cwd().child(app_name, module, 'tests', 'fixtures', '%s.yml' % yaml_file)
#
#    with open(yaml_file) as yaml_file:
#      fixtures = yaml.load(yaml_file.read())
#
#    def load_fixture(fixture_name):
#      if fixture_name in fixtures:
#        return fixtures[fixture_name]
#      else:
#        raise LookupError('%s doesnt exists in this fixture file' % fixture_name)
#
#    return load_fixture
#  return fixtures

import os
import yaml

def fixture(module):
  current_path = os.getcwd()
  fixture_path = "%s/tests/fixtures/%s.yml" % (current_path, module)

  with open(fixture_path) as fixture_file:
    yaml_fixture = yaml.load(fixture_file.read())

  def load_fixture(fixture_key):
    if fixture_key in yaml_fixture:
      return yaml_fixture[fixture_key]
    else:
      raise LookupError("Could not find key %s in file %s" % \
                       (fixture_key, fixture_path))

  return load_fixture



import sys

import json
from collections import OrderedDict
from functools import wraps

import inflect
from flask import Response, request

from sqlalchemy import and_
from sqlalchemy.orm import class_mapper
from sqlalchemy.ext.declarative import DeclarativeMeta

from finny.exceptions import HttpNotFound

from flask import current_app
from flask import request

#
#

class AlchemyEncoder(json.JSONEncoder):

  def _encode_declarative_meta(self, obj):
    # an SQLAlchemy class
    fields = {}
    for field in [x for x in dir(obj) if not x.startswith('_') and x != 'metadata']:
      data = obj.__getattribute__(field)

      if field in [ "query", "query_class" ]:
        continue

      try:
        json.dumps(data) # this will fail on non-encodable values, like other classes
        fields[field] = data
      except TypeError:
        fields[field] = None
    # a json-encodable dict
    return fields

  def default(self, obj):
    if isinstance(obj.__class__, DeclarativeMeta):
      return self._encode_declarative_meta(obj)

    return json.JSONEncoder.default(self, obj)

def serialize(func):
  @wraps(func)
  def endpoint_method(*args, **kwargs):
    response = func(*args, **kwargs)

    return Response(json.dumps(response, cls=AlchemyEncoder),  mimetype='application/json')

  return endpoint_method

class KaBoomError(Exception):
  status_code = 500

  def __init__(self, message, status_code=None, payload=None):
    Exception.__init__(self)
    self.message = message
    if status_code is not None:
      self.status_code = status_code
    self.payload = payload

  def to_dict(self):
    rv = dict(self.payload or ())
    rv['message'] = self.message
    return rv

def catch_it_all(func):
  @wraps(func)
  def endpoint_method(*args, **kwargs):
    if "DISABLED_ERROR_HANDLING" not in current_app.config:
      try:
        response = func(*args, **kwargs)
      except:
        current_app.config["SSENTRY"].captureException()
        raise KaBoomError("An error occured")
    else:
      response = func(*args, **kwargs)

    return response

  return endpoint_method

import inspect
from functools import partial

class ResourceBuilder(object):

  DAG = {}

  @classmethod
  def register(cls, klass):
    parent_klass = None
    if hasattr(klass, "__parent__"):
      parent_klass = klass.__parent__

    cls.DAG[klass] = parent_klass

  def __init__(self):
    pass

  def _get_parent_klasses(self, klass):
    def _get_parent(klass):
      if self.DAG[klass] == None:
        return []
      else:
        parent = self.DAG[klass]
        return [ parent ] + _get_parent(parent)

    return _get_parent(klass)

  def _resource_name(self, klass):

    if hasattr(klass, "route_base"):
      resource_name = klass.route_base
    else:
      resource_name = klass.__name__.lower()
      pos = resource_name.find("view")

      if pos == -1:
        raise AttributeError("Resource %s doesn't end in View" % resource_name)

      resource_name = resource_name[:pos]

    return resource_name

  def _make_show_url(self, klass):
    resource_name = self._resource_name(klass)
    """
    Resource Name can either be a single name OR _ separted words.
    If it's just a word for the param ID make it singular, for the
    underscore separated words make the last part singular
    """

    engine = inflect.engine()

    split_name = resource_name.split("_")
    entity_id  = engine.singular_noun(split_name[-1])

    if not entity_id:
      entity_id = split_name[-1]

    # ID must be singular
    return "/%s/<%s_id>" % (resource_name, entity_id)

  def _add_route(self, klass, url, method_name, http_verbs):
    methods = dict(inspect.getmembers(klass, predicate=inspect.ismethod))

    def call_method(method_name):
      instance = klass()

      method = getattr(instance, method_name)

      if hasattr(klass, "decorators"):
        for d in klass.decorators:
          method = d(method)

      return method

    if hasattr(klass, method_name) and method_name in methods:
      self.app.add_url_rule(url,
                            "%s::%s" % (klass.__name__, method_name),
                            call_method(method_name),
                            methods=http_verbs)
      # TODO: Why this?
      setattr(klass, "_orig_%s" % method_name, getattr(klass, method_name))
      setattr(klass, method_name, call_method(method_name))

  def _add_bypass_route(self, klass, url):
    # get all the available methods
    methods = dict(inspect.getmembers(klass, predicate=inspect.ismethod))

    def update_delete_bypass(ctx, **kwargs):
      http_verb = request.args.get('method')

      if http_verb == "put":
        return ctx._orig_update(**kwargs)
      elif http_verb == "delete":
        return ctx._orig_delete(**kwargs)
      else:
        raise AttributeError("Method is neither update or delete")

    # explicit put method in class
    setattr(klass, "update_delete_bypass", update_delete_bypass)

    def call_method(method_name):
      instance = klass()

      method = getattr(instance, method_name)

      if hasattr(klass, "decorators"):
        for d in klass.decorators:
          method = d(method)

      return method

    method_name = "update_delete_bypass"

    self.app.add_url_rule(url,
                          "%s::%s" % (klass.__name__, method_name),
                          call_method(method_name),
                          methods=["POST"])

    setattr(klass, "_orig_%s" % method_name, getattr(klass, method_name))
    setattr(klass, method_name, call_method(method_name))


  def _add_nested_route(self, app, klass, parent_klasses):
    """
    resource_name

    make_show_url(klass, name)
    """

    base_url = [ self._make_show_url(parent) for parent in reversed(parent_klasses) ]
    base_url = "".join(base_url)

    resource_name = self._resource_name(klass)
    resource_base = "%s/%s" % (base_url, resource_name)

    self._add_restful_routes(app, klass, resource_name, resource_base)

  def _add_restful_routes(self, app, klass, resource_name, resource_base):
    self.app = app

    engine = inflect.engine()
    split_name = resource_name.split("_")

    entity_id = engine.singular_noun(split_name[-1])
    if not entity_id:
      entity_id = split_name[-1]

    if "index" not in klass.except_methods:
      self._add_route(klass, resource_base, "index", ["GET"])

    if "create" not in klass.except_methods:
      self._add_route(klass, resource_base, "create", ["POST"])

    if "show" not in klass.except_methods:
      self._add_route(klass, resource_base + "/<%s_id>" % entity_id, "show", ["GET"])

    if "update" not in klass.except_methods:
      self._add_route(klass, resource_base + "/<%s_id>" % entity_id, "update", ["PUT"])

    if "delete" not in klass.except_methods:
      self._add_route(klass, resource_base + "/<%s_id>" % entity_id, "delete", ["DELETE"])

    self._add_bypass_route(klass, resource_base + "/<%s_id>" % entity_id)

  def _add_normal_route(self, app, klass):
    methods = dict(inspect.getmembers(klass, predicate=inspect.ismethod))

    resource_name = self._resource_name(klass)
    resource_base = "/" + resource_name

    self._add_restful_routes(app, klass, resource_name, resource_base)

  def build(self, app):
    for klass, klass_parent in self.DAG.iteritems():
      if klass_parent:
        parent_klasses = self._get_parent_klasses(klass)
        self._add_nested_route(app, klass, parent_klasses)
      else:
        self._add_normal_route(app, klass)

class Resource(object):

  decorators = [ serialize, catch_it_all ]

  @classmethod
  def register(cls):
    ResourceBuilder.register(cls)

class ModelResource(Resource):

  """
  Is nested than build the resources first
  """

  decorators = [ serialize, catch_it_all ]

  def index(self, **kwargs):
    if self.__is_nested():
      parents = self.__get_parents()
      models = self.__get_models(parents)
      model_ids = self.__get_model_ids(models)
      args_to_model = self.__get_args(parents, kwargs)

      query = self.db.session.query(self.model)

      for model in models.keys():
        query = query.join(getattr(model, models[model]))

      and_params = []
      for model in models.keys():
        pk_for_model = getattr(model, model_ids[model])
        key_name = args_to_model[model]
        and_params.append(pk_for_model.__eq__(kwargs[key_name]))

      items = query.filter(and_(*and_params)).all()
    else:
      items = self.model.query.all()

    return items

  def __get_args(self, parents, kwargs):
    engine = inflect.engine()

    args = {}

    for p in parents:
      name = p.__name__.lower()
      pos = name.find("view")
      arg = name[:pos]

      arg = arg.split("_")
      entity_id = engine.singular_noun(arg[-1])

      if not entity_id:
        entity_id = arg[-1]

      if ("%s_id" % entity_id) in kwargs:
        args[p.model] = "%s_id" % entity_id

    return args

  def __get_model_ids(self, models):
    return dict([ [m , class_mapper(m).primary_key[0].name] for m in models.keys() ])

  def __get_models(self, parents):
    models = dict([ [p, p.model] for p in parents ])

    to_assoc = OrderedDict()

    for p in range(len(parents)):
      if p == len(parents) - 1:
        continue

      parent = parents[p]
      associated = parents[p+1]
      assoc_name = models[associated].__tablename__

      if (assoc_name in models[parent].__backref__ and
          hasattr(models[parent], assoc_name)):
        to_assoc[models[parent]] = assoc_name
      else:
        raise LookupError("When calling a nested route you need to have a relationship backref from %s to %s named %s" % (model[parent], model[associated], assoc_name))

    return to_assoc

  def create(self, **kwargs):
    data = json.loads(request.data)
    item = self.model(**data)

    """
    You need the parent of the object that you will create
    """
    if self.__is_nested():
      parents = self.__get_parents()
      models = self.__get_models(parents)
      model_ids = self.__get_model_ids(models)
      args_to_model = self.__get_args(parents, kwargs)

      parent_view = parents[-2]
      parent_model = parent_view.model
      query = self.db.session.query(parent_model)

      join_models = models.keys()
      join_models.remove(parent_model)

      for model in join_models:
        query = query.join(getattr(model, models[model]))

      and_params = []
      for model in models.keys():
        pk_for_model = getattr(model, model_ids[model])
        key_name = args_to_model[model]
        and_params.append(pk_for_model.__eq__(kwargs[key_name]))

      parent_item = query.filter(and_(*and_params)).first()
      assoc_key = models[parent_model]
      assoc_list = getattr(parent_item, assoc_key)
      assoc_list.append(item)

    self.db.session.add(item)
    self.db.session.commit()

    return item

  def show(self, **kwargs):
    if self.__is_nested():
      parents = self.__get_parents()
      models = self.__get_models(parents)
      model_ids = self.__get_model_ids(models)
      args_to_model = self.__get_args(parents, kwargs)

      query = self.db.session.query(self.model)

      current_model_id = class_mapper(self.model).primary_key[0].name
      model_ids[self.model] = current_model_id

      for model in models.keys():
        query = query.join(getattr(model, models[model]))

      and_params = []
      for model in models.keys() + [ self.model ]:
        pk_for_model = getattr(model, model_ids[model])
        key_name = args_to_model[model]
        and_params.append(pk_for_model.__eq__(kwargs[key_name]))

      item = query.filter(and_(*and_params)).first()
    else:
      key = kwargs.keys()[0]
      item = self.model.query.get(kwargs[key])

    if not item:
      raise HttpNotFound({'error': 'Item not found in database'})

    return item

  def update(self, **kwargs):
    # TODO: Refactor this as the logic is the same for the show and update methods
    data = json.loads(request.data)
    if self.__is_nested():
      parents = self.__get_parents()
      models = self.__get_models(parents)
      model_ids = self.__get_model_ids(models)
      args_to_model = self.__get_args(parents, kwargs)

      query = self.db.session.query(self.model)

      current_model_id = class_mapper(self.model).primary_key[0].name
      model_ids[self.model] = current_model_id

      for model in models.keys():
        query = query.join(getattr(model, models[model]))

      and_params = []
      for model in models.keys() + [ self.model ]:
        pk_for_model = getattr(model, model_ids[model])
        key_name = args_to_model[model]
        and_params.append(pk_for_model.__eq__(kwargs[key_name]))

      item = query.filter(and_(*and_params)).first()
    else:
      key = kwargs.keys()[0]
      item = self.model.query.get(kwargs[key])

    if not item:
      raise HttpNotFound({'error': 'Item not found in database'})

    for field in data:
      if hasattr(item, field):
        setattr(item, field, data[field])

    self.db.session.add(item)
    self.db.session.commit()

    return item

  def delete(self, **kwargs):
    # TODO: Refactor this as the logic is the same for the show and update methods
    if self.__is_nested():
      parents = self.__get_parents()
      models = self.__get_models(parents)
      model_ids = self.__get_model_ids(models)
      args_to_model = self.__get_args(parents, kwargs)

      query = self.db.session.query(self.model)

      current_model_id = class_mapper(self.model).primary_key[0].name
      model_ids[self.model] = current_model_id

      for model in models.keys():
        query = query.join(getattr(model, models[model]))

      and_params = []
      for model in models.keys() + [ self.model ]:
        pk_for_model = getattr(model, model_ids[model])
        key_name = args_to_model[model]
        and_params.append(pk_for_model.__eq__(kwargs[key_name]))

      item = query.filter(and_(*and_params)).first()
    else:
      key = kwargs.keys()[0]
      item = self.model.query.get(kwargs[key])

    if not item:
      raise HttpNotFound({'error': 'Item not found in database'})

    self.db.session.delete(item)
    self.db.session.commit()

    return {'success': 'Item was deleted!'}

  def __is_nested(self):
    return hasattr(self, "__parent__")

  def __get_parents(self, **kwargs):
    """
    From base to root
    """
    def _get_parent(klass):
      if not hasattr(klass, "__parent__"):
        return []
      else:
        parent = klass.__parent__
        return [ parent ] + _get_parent(parent)

    return _get_parent(self)[::-1] + [ self.__class__ ]



from setuptools import setup, find_packages

requires = [ "pyyaml>=3.10" ]

setup(name="finny",
      version="0.3.9",
      platforms='any',
      packages = find_packages(),
      include_package_data=True,
      install_requires=requires,
      author = "Bogdan Gaza",
      author_email = "bc.gaza@gmail.com",
      url = "https://github.com/hurrycane/finny",
      description = """Finny is the act of being skinny and fat at the same time.
                     Basic structure for an api-centry approach to Flask - that is both fat in skinny,
                     with basic and augmented support over some popular Flask libs""",
      entry_points = {'console_scripts': [ 'finny = finny.runner:execute_from_cli' ]},
      test_requirements = [],
      classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Topic :: System :: Networking',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
      ]
)
